<?php

namespace backend\modules\product\models;

use yeesoft\behaviors\MultilingualBehavior;
use yeesoft\db\ActiveRecord;
use yeesoft\models\User;
use Yii;
use yii\behaviors\BlameableBehavior;
use yii\behaviors\SluggableBehavior;
use yii\behaviors\TimestampBehavior;

/**
 * This is the model class for table "product".
 *
 * @property int $id
 * @property string $slug
 * @property string $image
 * @property string $image_header
 * @property string $icon_1
 * @property string $icon_2
 * @property string $icon_3
 * @property int $status 0-pending,1-published
 * @property int $on_homepage_main_slider
 * @property int $on_homepage_second_slider
 * @property int $is_online
 * @property int $published_at
 * @property int $created_at
 * @property int $updated_at
 * @property int $created_by
 * @property int $updated_by
 * @property int $revision
 * @property string $color
 * @property string $view
 * @property string $layout
 * @property string $type
 * @property string $url
 *
 * @property SubProduct[] $subProducts
 */
class Product extends ActiveRecord
{

    const STATUS_PENDING = 0;
    const STATUS_PUBLISHED = 1;


    /**
     * {@inheritdoc}
     */
    public static function tableName()
    {
        return 'product';
    }

    /**
     * {@inheritdoc}
     */
    public function rules()
    {
        return [
            [['title', 'brief'], 'required'],
            [['created_by', 'updated_by', 'status', 'revision', 'on_homepage_main_slider', 'on_homepage_second_slider', 'is_online'], 'integer'],
            [['slug', 'image', 'icon_1', 'icon_2', 'icon_3', 'view', 'layout', 'type', 'image_header', 'url', 'sub_title', 'brief'], 'string', 'max' => 255],
            [['color'], 'string', 'max' => 7],
            ['slug', 'unique'],
            [['created_at', 'updated_at', 'start_date', 'end_date'], 'safe'],
            [['title', 'content', 'view', 'layout'], 'string'],
            ['published_at', 'date', 'timestampAttribute' => 'published_at', 'format' => 'yyyy-MM-dd'],
            ['published_at', 'default', 'value' => time()],


        ];
    }

    /**
     * @inheritdoc
     */
    public function behaviors()
    {
        return [
            TimestampBehavior::className(),
            BlameableBehavior::className(),
            'sluggable' => [
                'class' => SluggableBehavior::className(),
                'attribute' => 'title',
            ],
            'multilingual' => [
                'class' => MultilingualBehavior::className(),
                'langForeignKey' => 'product_id',
                'tableName' => "{{%product_lang}}",
                'attributes' => [
                    'title', 'sub_title', 'content', 'brief'
                ]
            ],
        ];
    }

    /**
     * {@inheritdoc}
     */
    public function attributeLabels()
    {
        return [
            'id' => Yii::t('site', 'ID'),
            'slug' => Yii::t('site', 'Slug'),
            'image' => Yii::t('site', 'Home Page Image'),
            'image_header' => Yii::t('site', 'Header Image'),
            'icon_1' => Yii::t('site', 'Icon 1'),
            'icon_2' => Yii::t('site', 'Icon 2'),
            'icon_3' => Yii::t('site', 'Icon 3'),
            'status' => Yii::t('site', 'Status'),
            'on_homepage_main_slider' => Yii::t('site', 'On Homepage Main Slider'),
            'on_homepage_second_slider' => Yii::t('site', 'On Homepage Second Slider'),
            'published_at' => Yii::t('site', 'Published At'),
            'created_at' => Yii::t('site', 'Created At'),
            'updated_at' => Yii::t('site', 'Updated At'),
            'created_by' => Yii::t('site', 'Created By'),
            'updated_by' => Yii::t('site', 'Updated By'),
            'revision' => Yii::t('site', 'Revision'),
            'view' => Yii::t('site', 'View'),
            'layout' => Yii::t('site', 'Layout'),
            'url' => Yii::t('site', 'URL'),
        ];
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getProductLangs()
    {
        return $this->hasMany(ProductLang::className(), ['product_id' => 'id']);
    }

    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubProducts()
    {
        return $this->hasMany(SubProduct::className(), ['parent_id' => 'id']);
    }
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getSubProductsWithVoucher()
    {
        return $this->hasMany(SubProduct::className(), ['parent_id' => 'id'])->innerJoinWith('services')->andWhere(['is_voucher' => true]);
    }
    /**
     * @return \yii\db\ActiveQuery
     */
    public function getServices()
    {
        return $this->hasMany(Service::className(), ['parent_id' =>'id'])->viaTable('sub_product', ['parent_id' => 'id']);
    }

    /**
     * {@inheritdoc}
     * @return ProductQuery the active query used by this AR class.
     */
    public static function find()
    {
        return new ProductQuery(get_called_class());
    }


    public function save($runValidation = true, $attributeNames = null)
    {
        \Yii::$app->cache->flush();
        return parent::save($runValidation, $attributeNames);
    }


    public function getAuthor()
    {
        return $this->hasOne(User::className(), ['id' => 'created_by']);
    }

    public function getUpdatedBy()
    {
        return $this->hasOne(User::className(), ['id' => 'updated_by']);
    }

    public function getPublishedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->published_at);
    }

    public function getCreatedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->created_at);
    }

    public function getUpdatedDate()
    {
        return Yii::$app->formatter->asDate(($this->isNewRecord) ? time() : $this->updated_at);
    }

    public function getPublishedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->published_at);
    }

    public function getCreatedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->created_at);
    }

    public function getUpdatedTime()
    {
        return Yii::$app->formatter->asTime(($this->isNewRecord) ? time() : $this->updated_at);
    }

    public function getPublishedDatetime()
    {
        return "{$this->publishedDate} {$this->publishedTime}";
    }

    public function getCreatedDatetime()
    {
        return "{$this->createdDate} {$this->createdTime}";
    }

    public function getUpdatedDatetime()
    {
        return "{$this->updatedDate} {$this->updatedTime}";
    }

    public function getStatusText()
    {
        return $this->getStatusList()[$this->status];
    }


    public function getRevision()
    {
        return ($this->isNewRecord) ? 1 : $this->revision;
    }

    public function updateRevision()
    {
        $this->updateCounters(['revision' => 1]);
    }

    /**
     * getTypeList
     * @return array
     */
    public static function getStatusList()
    {
        return [
            self::STATUS_PENDING => Yii::t('yee', 'Pending'),
            self::STATUS_PUBLISHED => Yii::t('yee', 'Published'),
        ];
    }

    /**
     * getTypeList
     * @return array
     */
    public static function getColorList()
    {
        return [
            '' => '',
            'orange' => 'Orange',
            'red' => 'Red',
            'blue' => 'Blue',
            'green' => 'Green',
        ];
    }

    /**
     * getTypeList
     * @return array
     */
    public static function getTypeList()
    {
        return [
            '' => '',
            'IndividualProducts' => Yii::t('site', 'Individual Products'),
            'CorporateProducts' => Yii::t('site', 'Corporate Products'),
        ];
    }
    /**
     * getStatusOptionsList
     * @return array
     */
    public static function getStatusOptionsList()
    {
        return [
            [self::STATUS_PENDING, Yii::t('yee', 'Pending'), 'default'],
            [self::STATUS_PUBLISHED, Yii::t('yee', 'Published'), 'primary']
        ];
    }

    /**
     *
     * @inheritdoc
     */
    public static function getFullAccessPermission()
    {
        return 'fullPageAccess';
    }

    /**
     *
     * @inheritdoc
     */
    public static function getOwnerField()
    {
        return 'created_by';
    }

    /**
     * getStatusOptionsList
     * @return string
     */
    public function __toString()
    {
        return $this->title;
    }

}
